let _ =
  Units.initialize ();
  Accountdbase.load_initial_files ();
  Meets.process ();
  Splits.process ();
  Vaccounts.process ();
  Vaccounts.process_late ();
  Printaccount.create_balance_sheets ();
  Groupsummary.emit ();
  print_endline "Processed successfully."

