type amount
type default_units_amount

val create : int (* default units *) -> string -> amount

val to_string : amount -> string

val is_negative : amount -> bool
val zero_default : default_units_amount
val zero : amount
val abs_default : default_units_amount -> default_units_amount
val add : amount -> default_units_amount -> default_units_amount
val add_default : default_units_amount -> default_units_amount -> default_units_amount
val negate : amount -> amount
val negate_default : default_units_amount -> default_units_amount
val subtract_default : default_units_amount -> default_units_amount ->
		       default_units_amount
val scale_default : float -> float -> default_units_amount ->
                    default_units_amount
val to_string_default : default_units_amount -> string
val non_negative_default : default_units_amount -> bool
val very_negative_default : default_units_amount -> bool
val default_to_normal : default_units_amount -> amount
val is_zero : amount -> bool

