(* Handling of virtual accounts. *)

type vaccount_op = Add | Subtract
type vaccount_entry = Account of string | Group of string

type vaccount = { name : string;
                  group : string;
                  entries : (vaccount_op * vaccount_entry) list }

