body		{ font-family: sans-serif; margin-left: 5%; margin-right: 5% }

td.grandtotalp	{ background-color: #00ff00 }
td.grandtotalm	{ background-color: #ff8800 }
td.grandtotaln	{ background-color: #ff0000 }
td.groupsummary	{ background-color: #cccccc }

tr.incomeexp	{ background-color: #dddddd }
tr.checkpoint	{ background-color: #aaaaaa }
tr.fixme	{ background-color: #ffdd00 }
tr.grandtotalp	{ background-color: #00ff00 }
tr.grandtotaln	{ background-color: #ff0000 }
tr.actual	{ background-color: #cccccc }
tr.notional     { background-color: #eeeeee }
tr.header	{ background-color: #6666ff }
