(* Miscellaneous functions. *)

val get_root_directory : unit -> string
val open_config_file : string -> in_channel
val open_config_file_fail_ok : string -> in_channel
val iter_files : string -> (string -> in_channel -> unit) -> unit

val format_money : int -> string
val round : float -> float

val verbose : string -> unit
val fail : string -> 'a

